/*
 * Copyright 2015 Djembe.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package md.djembe.beans;

import java.io.Serializable;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;
import javax.faces.event.ActionEvent;
import javax.faces.event.ComponentSystemEvent;
import javax.servlet.ServletContext;

import md.djembe.cache.ApplicationCache;
import md.djembe.cache.DataCache;
import md.djembe.entities.ArticleEntity;
import md.djembe.entities.SettingEntity;
import md.djembe.entities.TemplateEntity;
import md.djembe.enums.WebAppProperties;
import md.djembe.facades.ArticleFacade;
import md.djembe.facades.InstallFacade;
import md.djembe.facades.SettingFacade;
import md.djembe.facades.TemplateFacade;
import md.djembe.persistence.PersistenceUtil;

import org.apache.log4j.Logger;

/**
 * The main application scoped bean
 */
@ManagedBean(name = "applicationBean")
@ApplicationScoped
public class ApplicationBean extends AbstractBean implements Serializable {

	private static final long serialVersionUID = -1L;
	
	/** The Logger */
	private static final Logger LOGGER = Logger.getLogger(ApplicationBean.class);
	
	/** The scheduler to run various periodic service */
	private ScheduledExecutorService scheduler;
	
	/** The number of thread to run the background tasks */
	private int corePoolSize = 2;
	
	/** The entities are cached as long as this bean is alive */
	private List<ArticleEntity> articles;
	
	/** The Web content element */
	private Map<String, String> webContent;
	
	/** The public pages template file */
	private TemplateEntity publicTemplate;
	

	@PostConstruct
	public void init() {
		final String persistenceUnitName = getExternalContext().getInitParameter("persistenceUnitName");
		PersistenceUtil.setPersistenceUnitName(persistenceUnitName);
		
		//if not set, initialise application
		initialiseApplicationContent( );

		loadApplicationContent( );

		/** Initialise the various background tasks */
		ThreadFactory THREAD_FACTORY = new TaskThreadFactory();
		scheduler = Executors.newScheduledThreadPool(corePoolSize, THREAD_FACTORY);
		scheduleMailerTask();
	}

	@PreDestroy
	public void cleanUp() {
		scheduler.shutdownNow();
	}
	
	/**
	 * Load the application data
	 */
	public void loadApplicationContent(){
		retrievePublicTemplate( );
		retrieveAllArticle( );
		refreshApplicationCache( );
	}

	/**
	 * Refresh the application data cache by reloading the data in the database
	 */
	public void refreshApplicationContent( ActionEvent actionEvent) {
		if(LOGGER.isDebugEnabled()){
			LOGGER.debug("Reloading the Application data from the DB.");
		}
		loadApplicationContent( );
	}
	
	public void initialiseApplicationContent() {
		if (! isInstallComplete()) {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("New install, loading data...");
			}
			InstallFacade installFacade = new InstallFacade();
			installFacade.loadInitialData( ( ServletContext ) getExternalContext().getContext());
		}
	}

	private boolean isInstallComplete() {
		SettingEntity installComplete = new SettingFacade().findByName("installComplete");
		if (installComplete != null) {
			return Boolean.valueOf(installComplete.getSettingValue());
		}
		return false;
	}

	/**
	 * Retrieve the public pages template
	 */
	public void retrievePublicTemplate() {
		publicTemplate = new TemplateFacade().getActiveTemplate();
	}

	/**
	 * Retrieve all the articles stored in the DB
	 */
	public void retrieveAllArticle() {
		List<ArticleEntity> resultList = (List<ArticleEntity>) new ArticleFacade()
				.getAllBlogArticle();
		articles = resultList;
	}
	
	public void refreshApplicationCache(){
		DataCache.INSTANCE.refreshCachedData();
	}
	

	/**
	 * Load the dynamic pages. The content of the page is stored in the DB.
	 * The dynamically generated page will be displayed in views.xhtml
	 * 
	 * @param event The System event
	 */
    public void loadPage( ComponentSystemEvent event ){
        
        final String requestedPage = (String) getExternalContext( ).getRequestMap( ).get( "pageName" );
        
        if(LOGGER.isDebugEnabled()){
        	LOGGER.debug( String.format( "Forwarding to requested page: %s" , requestedPage ) );
        }
        
//        getFacesContext( )
//                .getApplication( )
//                .getNavigationHandler( )
//                .handleNavigation( getFacesContext( ) , null , requestedPage + WebAppProperties.DYNAMIC_PAGES.getPropertyValue( ) );

    }
    
    public String getPageContent(){
    	final String requestedPage = (String) getExternalContext( ).getRequestMap( ).get( "pageName" );
    	
    	if(LOGGER.isDebugEnabled()){
        	LOGGER.debug( String.format( "Loading the page content: %s" , requestedPage ) );
        }
    	
    	final String entry = ApplicationCache.getEntry( WebAppProperties.WEBPAGES , requestedPage );
    	
    	return entry;
    }

	/**
	 * Schedule the mailer task. The task will run at the scheduled period no
	 * matter how long the task takes.
	 */
	public void scheduleMailerTask() {
		String mailerTaskDelay = ApplicationCache.getEntry( WebAppProperties.SETTINGS, "mailerTaskDelay" );
		mailerTaskDelay = ( mailerTaskDelay != null ? mailerTaskDelay : "0" ) ; 
		int delay = Integer.parseInt(mailerTaskDelay);
		try {
			delay = (delay > 60 ? delay : 60000);
			// scheduler.scheduleAtFixedRate( new MailerTask( ) , 0 , delay ,
			// TimeUnit.SECONDS ) ;
		} catch (NumberFormatException nfe) {
			// do nothing
		}
	}

	// ###############################################################
	// GETTER AND SETTERS
	// ###############################################################

	public List<ArticleEntity> getArticles() {
		return articles;
	}

	public Map<String, String> getWebContent() {
		return webContent;
	}

	public TemplateEntity getPublicTemplate() {
		return publicTemplate;
	}

	public void setPublicTemplate(TemplateEntity publicTemplate) {
		this.publicTemplate = publicTemplate;
	}


	// ###############################################################
	// INNER CLASSES
	// ###############################################################

	/**
	 * Initialise and configure the tread
	 */
	private static final class TaskThreadFactory implements ThreadFactory {
		AtomicInteger count = new AtomicInteger();

		@Override
		public Thread newThread(Runnable r) {
			Thread thread = new Thread(r);
			thread.setName(String.format("Background Thread %s",
					count.getAndIncrement()));
			// thread.setDaemon( true );
			return thread;
		}

	}

}
