/*
 * Copyright 2015 Djembe.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package md.djembe.beans;

import java.io.Serializable;
import java.util.List;
import java.util.Locale;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.validation.constraints.NotNull;

import md.djembe.entities.UserEntity;
import md.djembe.enums.UserTypes;
import md.djembe.exceptions.UserRegistrationException;
import md.djembe.exceptions.UserRetrievalException;
import md.djembe.facades.UserFacade;



@ManagedBean
@SessionScoped
public class UserBean extends AbstractBean implements Serializable{
    
	private static final long serialVersionUID = 1L;

	@NotNull
    private String username;
    
    @NotNull
    private String email;
    
    @NotNull
    private String password;
    
    @NotNull
    private String confirmPassword;
    
    private UserTypes userType;
    
    private UserEntity user;
    
    private List<UserEntity> userList;
    
    private String userLanguage ;
    
    @PostConstruct
    public void init( ) {
        retrieveUsers( );
        String language = getExternalContext().getRequestLocale().getLanguage();
        System.out.println("User language == " + language);
        userLanguage = language.equals( new Locale( "fr" ).getLanguage( ) ) ? "FR" : "EN" ;
    }
    
    
    //############################################
    // CRUD
    //############################################   
    
   
    /**
     * Add a new entry to the database
     * @return
     */
    public String createNewUserEntity( ) {
        try {
            new UserFacade( ).createNewUser( username , email ,
                                             password , confirmPassword , userType );
            retrieveUsers( );
            resetFields( );
        }catch ( UserRegistrationException ure ) {
            logError(String.format( "Error creating a new user. username : %s, email : %s", username, email ), ure ) ;
           // addToAlertEmail( ure );
        }
        return "pretty:listUsers";
    }
    
    /**
     * navigate to the view UserEntity page
     * @return
     */
    public String displayUserEntity( UserEntity user ) {
        this.user = user;
        return "pretty:viewUser";
    }
    
    /**
     * Select the UserEntity for edit
     * @return
     */
    public String editUserEntity( UserEntity user ) {
        this.user = user;
        return "pretty:editUser";
    }
    
    /**
     * Save the changes made to the UserEntity
     * @return
     */
    public String updateUserEntity( UserEntity user ) {
        logInfo( String.format( "Updating user details, username : %s", user.getUsername( ) ) );
        new UserFacade( ).edit( user );
        retrieveUsers( );
        return "pretty:listUser";
    }
    
    /**
     * Remove the UserEntity from the database
     * @return
     */
    public String deleteUserEntity( UserEntity user ) {
        logInfo( String.format( "Delete user, username : %s", user.getUsername( ) ) );
        new UserFacade( ).remove( user );
        retrieveUsers( );
        return "pretty:listUser";
    }
    
    /**
     * Navigate to the create new UserEntity page
     * @return
     */
    public String addNewUserEntity( ) {
        resetFields( );
        return "pretty:newUser";
    }
    
    /**
     * navigate to the view UserEntity page
     * @return
     */
    public String previewUserEntity( UserEntity user ) {
        this.user = user;
        return "pretty:viewUser";
    }
    
    
     private void retrieveUsers( ) {
         try {
             userList = new UserFacade( ).findAll( );
         }catch( UserRetrievalException ure ) {
             logError( "Error retrieving users from the DB." , ure );
         }
    }
    
    
    private void resetFields( ) {
        username = null;
        email = null;
        password = null;
        confirmPassword = null;
        userType = null;
    }

  
    
    
    // ###############################################################
    // GETTER AND SETTERS
    // ###############################################################
    
    public String getUsername( ) {
        return username;
    }

    public void setUsername( String username ) {
        this.username = username;
    }

    public String getEmail( ) {
        return email;
    }

    public void setEmail( String email ) {
        this.email = email;
    }

    public String getPassword( ) {
        return password;
    }

    public void setPassword( String password ) {
        this.password = password;
    }

    public String getConfirmPassword( ) {
        return confirmPassword;
    }

    public void setConfirmPassword( String confirmPassword ) {
        this.confirmPassword = confirmPassword;
    }

    public UserTypes getUserType( ) {
        return userType;
    }

    public void setUserType( UserTypes userType ) {
        this.userType = userType;
    }

    public UserEntity getUser( ) {
        return user;
    }

    public List<UserEntity> getUserList( ) {
        return userList;
    }


	public String getUserLanguage() {
		return userLanguage;
	}


	public void setUserLanguage(String userLanguage) {
		this.userLanguage = userLanguage;
	}
    
    
    
    
}
