/*
 * Copyright 2015 Djembe.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package md.djembe.cache;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;


/**
 * Cache of data backed by the DB
 * 
 * @author M. Diallo
 *
 */
public enum DataCache{
    
    INSTANCE;
    
    private static final Logger LOGGER = Logger.getLogger( DataCache.class );
    
    private DataCache() {
        // nothing
    }
    
    private volatile Map<String, Cache> cacheMap = new HashMap<>( );
    
    
    public String getEntry( final String content , final String item ){

        final Map<String , String> applicationEntries = getEntries( content );
        if( applicationEntries == null ){            
            return null;
        }
        return applicationEntries.get( item );
    }
    
    
    private Map<String , String> getEntries( final String content ){
        
        Cache cache = getCacheMap( ).get( content );
        
        if( cache == null ){
            reloadApplicationCache( content );
        }
        cache = getCacheMap( ).get( content );
        return cache == null ? null : cache.getCacheEntry( );
    }
    
    
    public void reloadApplicationCache( final String tableName ) {
        
        final Cache newCache = loadCache( tableName );

        final Map<String, Cache> newCacheMap = new HashMap<>( cacheMap );
        newCacheMap.put( tableName , newCache );
        cacheMap = newCacheMap ;
    }
    
    
    private Cache loadCache( final String tableContent ) {

        if( LOGGER.isDebugEnabled( ) ) {
            LOGGER.debug( String.format( "Building entry cache for %s" , tableContent ) );
        }

        Map<String , String> cacheEntries = DataLoader.loadEntries( tableContent );

        Cache cache = buildCache( tableContent , cacheEntries );

        if( LOGGER.isDebugEnabled( ) ) {
            LOGGER.debug( String.format( "Cache built for %s with %d entries" , tableContent, cache.getSize( ) ) );
        }

        return cache;
    }
    
    
    private Cache buildCache( final String content, final Map<String, String> entries ) {
        Cache entryCache = new Cache( content , entries );
        return entryCache;
    }
    
    
    public void refreshCachedData( ){
    	for(String content : cacheMap.keySet()){
    		reloadApplicationCache(content);
    	}
    }
    
    public Map<String , Cache> getCacheMap( ){
        return cacheMap;
    }


    private static final class Cache{
        
        private final String content;
        private final Map<String, String> cacheEntry;
        
        public Cache( String content , Map<String , String> cacheEntry ){
            this.content = content;
            this.cacheEntry = cacheEntry;
        }

        public Map<String , String> getCacheEntry( ){
            return cacheEntry;
        }
        
        public int getSize() {
            return cacheEntry.size( );
        }

        @Override
        public String toString( ){
            return String.format("Cache [content=%s, cacheEntry=%s", content, cacheEntry );
        }
                
    }
    
    
    private static final class DataLoader{
        
        @SuppressWarnings( "unchecked" )
        public static Map<String , String> loadEntries( final String tableName ){

            Map<String , String> cacheEntries = null;
            try {
                
                final String className = "md.djembe.facades." + StringUtils.capitalize( tableName )+ "Facade";
                
                Class<?> entityFacadeClass = Class.forName( className );
                
                Object entityFacadeInstance = entityFacadeClass.newInstance( );
                
                Method initCacheMethod = entityFacadeClass.getMethod( "initCache" );
                
                cacheEntries = (Map<String , String>) initCacheMethod.invoke( entityFacadeInstance );
               
            }
            catch( Exception e ) {
                LOGGER.error( String.format(
                                "Unable to load cache for table [%s]; Error: \n%s" ,
                                tableName , e ) );

            }
            return cacheEntries;

        }
    }

}
