/*
 * Copyright 2015 Djembe.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package md.djembe.facades;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;

import md.djembe.crypto.EncryptionManagerFactory;
import md.djembe.crypto.Encryptor;
import md.djembe.entities.UserEntity;
import md.djembe.entities.UserGroupEntity;
import md.djembe.enums.UserRoles;
import md.djembe.enums.UserTypes;
import md.djembe.exceptions.UserLoginException;
import md.djembe.exceptions.UserRegistrationException;
import md.djembe.exceptions.UserRetrievalException;
import md.djembe.persistence.PersistenceUtil;

public class UserFacade extends AbstractFacade<UserEntity> {

    private Encryptor encryptor;

    public UserFacade( ) {
        super( UserEntity.class );
    }

    @Override
    protected EntityManager getEntityManager( ) {
        return PersistenceUtil.getEntityManager( );
    }

    /**
     * Create a new entity user and persist to database.
     * 
     * @param username
     * @param password
     * @param userType
     */
    public boolean createNewUser( final String username , final String email ,
                                final String userPassword , final String confirmUserPassword ,
                                final UserTypes userType ) throws UserRegistrationException {

        try {
        	
        	// check both password are the same
        	if( !userPassword.equals( confirmUserPassword ) ) {
                return false;
            }            
            
            UserEntity user = new UserEntity( );
            user.setUsername( username );
            user.setEmail( email );
            user.setUsertype( userType );
            user.setCreationDate( System.currentTimeMillis( ) );
            user.setActivated( Boolean.FALSE );
            
            encryptor = new EncryptionManagerFactory( ).createBCryptEncryptor( );
            // encode the password
            String hashUserPassword = encryptor.hashPassword( userPassword );
            user.setUserPassword( hashUserPassword );
            
            create( user );

            UserGroupEntity groupEntity = new UserGroupEntity( );
            groupEntity.setUserEntity( user );
            
            UserRoles userRole = ( userType.equals( UserTypes.ADMIN ) ? UserRoles.ADMIN : UserRoles.USER ) ;
            
            groupEntity.setUserrole( userRole );
            new UserGroupFacade( ).create( groupEntity );

            return true;

        }
        catch( Throwable throwable ) {
            throw new UserRegistrationException( throwable );
        }

    }

    /**
     * Retrieve a user from database given the username and password
     * 
     * @param username
     * @param plainPassword
     * @throws NoResultException
     *             if no user found.
     * @return
     */
    public UserEntity getUser( final String username , final String plainPassword ) 
                                                    throws UserLoginException , UserRetrievalException {
        UserEntity user = null;
        try {
            //retrieve the user
            user = ( UserEntity ) getEntityManager( ).createQuery(
                                    "SELECT u FROM UserEntity u WHERE u.username = :username" )
                                    .setParameter( "username" , username ).getSingleResult( );
            if( user != null ) {
            	
            	encryptor = new EncryptionManagerFactory( ).createBCryptEncryptor( );
                //compare password
                boolean matchingPassword = encryptor.isMatchingPassword(
                        plainPassword , user.getUserPassword( ) );

                if( !matchingPassword ) {
                    throw new UserLoginException( "Credentials do not match" );
                }
            }
        }
        catch( Throwable throwable ) {
            throw new UserRetrievalException( "Error login to the platform" );
        }
        return user;
    }

    /**
     * Retrieve all the users from the DB
     * 
     * @return
     */
    @SuppressWarnings( "unchecked" ) 
    public List<UserEntity> getAllUsers( ) {
        List<UserEntity> resultList = new ArrayList<UserEntity>( );
        try {
            resultList = (List<UserEntity>) getEntityManager( )
                                            .createQuery( "SELECT u FROM UserEntity u" )
                                            .getResultList( );
        } catch( Throwable throwable ) {
             throw new UserRetrievalException( throwable );
         }
        return resultList;
    }

}
