/*
 * Copyright 2015 Djembe.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package md.djembe.resources;

import java.io.IOException;
import java.net.URL;

import javax.faces.application.ResourceHandler;
import javax.faces.application.ResourceHandlerWrapper;
import javax.faces.application.ViewResource;
import javax.faces.context.FacesContext;

import md.djembe.cache.ApplicationCache;
import md.djembe.enums.WebAppProperties;
import md.djembe.exceptions.ResourceNotFoundException;

import org.apache.log4j.Logger;

public class DynamicResourceHandler extends ResourceHandlerWrapper {
	
    /** The Logger */
	private static final Logger LOGGER = Logger.getLogger(DynamicResourceHandler.class);
	/** The dynamic pages format*/
	private static final String DYNAMIC_PAGES_FORMAT = WebAppProperties.DYNAMIC_PAGES.getPropertyValue( );
	/** The view where the dynamic pages display */
	private static final String[] LOCATIONS = new String[] { "views.xhtml", "WEB-INF", "_admin/", "adminLogin" } ;

	private ResourceHandler wrapped;

	public DynamicResourceHandler(ResourceHandler wrapped) {
		this.wrapped = wrapped;
	}

    @Override
    public ViewResource createViewResource( FacesContext context , String resourceName ) {
    	
    	if( resourceName.startsWith("/admin/")){
    		resourceName = redirectToAdminUrL(resourceName);
    	}

        if( LOGGER.isDebugEnabled( ) ){
            LOGGER.debug( String.format( "Processing requested resource : %s" ,resourceName ) );
        }

        String viewName = resourceName;
        
        if( isValidRequest( viewName ) ) {
            
            viewName = resourceName.substring( 1 , resourceName.indexOf( DYNAMIC_PAGES_FORMAT ) );
        
            if( LOGGER.isDebugEnabled( ) ) {
                LOGGER.debug( String.format( "Processing requested page : %s" , viewName ) );
            }
            
            String entry = ApplicationCache.getEntry( WebAppProperties.WEBPAGES , viewName );
            
            if( entry != null ) {
                try {
                	
                	// create a unique spec value so that fresh changes can be picked up
                	final String spec = "string://" + viewName + "_" + System.currentTimeMillis() ;
                	
                    final URL url = new URL( null , spec , new DynamicProtocolHandler( entry ) );
    
                    if( LOGGER.isDebugEnabled( ) ) {
                        LOGGER.debug( String
                                .format( "Match found for page : %s, dynamic URL : %s" , viewName, url ) );
                    }
    
                    return new ViewResource( ) {
    
                        @Override
                        public URL getURL( ){
                            return url;
                        }
                    };
                }
                catch( IOException e ) {
                    LOGGER.error( String
                            .format( "Error processing the requested resource %s; Error was \n%s" ,
                                    resourceName , e ) );
                }
            }else {
                LOGGER.error( String.format( "The requested resource '%s' was not found" , viewName) );
                try {
                    context.getExternalContext( ).responseSendError( 404 , String.format( "The requested resource '%s' was not found" , viewName) );
                }
                catch( IOException e ){
                    LOGGER.error( "Error sending 404 response code." );
                    throw new ResourceNotFoundException( "Error sending 404 response code." ) ;
                }
            }
            
        }
        
        return super.createViewResource( context , resourceName );
    }

	@Override
	public ResourceHandler getWrapped() {
		return wrapped;
	}
	
	
    /**
     * Determine if the request is valid for the dynamic pages
     * @param viewName
     * @return true if the request is valid for a dynamic page
     */
    private static boolean isValidRequest( final String viewName ) {        
        for( String location : LOCATIONS ) {
            if( viewName.contains( location ) ) {
                return false;
            }
        }
        return viewName.contains( DYNAMIC_PAGES_FORMAT  );
    }
    
    // seems the redirection is done internally, without going through the request filter
    private static String redirectToAdminUrL( final String resourceName ){    	
    	return new StringBuilder(resourceName).insert(1, '_').toString() ;
    }
    
	

}
